/**
* \file state.h
**/

#ifndef _IGAMESTATE_HPP_INCLUDED
#define _IGAMESTATE_HPP_INCLUDED

#include <string>
#include <irrlicht.h>
#include <boost/shared_ptr.hpp>
#include <boost/enable_shared_from_this.hpp>
#include "../bcCompileConfig.hpp"
#include "../def.hpp"
#include <mirrpp/IPostProcessingHandler.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/variant.hpp>
#include <set>
#include <map>
//#include "../game_engine.hpp"
//#define ENGINE CGameEngine

class IStateManager;
class CGameEngine;
class EffectHandler;

//#define DECLARE_GAME_SINGLETON(ClassName) template<> ClassName* IGameStateSingleton< ClassName >::Instance = 0;
#define DECLARE_GAME_SINGLETON(ClassName) ;

//#define DECLARE_GAME_STATE(ClassName)   class ClassName : public IBasicGameState, public IGameStateSingleton<ClassName>
#define DECLARE_GAME_STATE(ClassName)   class ClassName : public IGameState
//, public irr::IEventReceiver, public
//#define DECLARE_GAME_STATE(ClassName)   class ClassName : public IGameState


//#declare GAMESTATE_LOADING_FUNCTION() load

//#define CONSTRUCTOR_PROTOTYPE                const std::string& name \
//                                                    , const bool& showCursor \
//                                                    , const bool& draw2d \
//                                                    , const bool& draw3d \
//                                                    , const bool& cloneScene \
//                                                    , const bool& displayScreenshotAsBackground
//


// TODO declarer le constructeur
//#define INNER_STATE_DECLARATION(className) friend class IGameStateSingleton<className>; className();
#define INNER_STATE_DECLARATION(className) public: className();


//push_back
//#ifdef DEBUG
//#define ADD_REQUIRED(pointer,name) Required.insert( &(pointer) );
//#else
#define ADD_REQUIRED(pointer,name) Required.insert( std::make_pair( &(pointer), name) );

namespace irr
{
    namespace scene
    {
        class ISceneManager;
    }
}


class IPostProcessingHandler;
struct lua_State;



/**
we can now use  "shared_from_this()" to get a shared_ptr of "this"
**/
//template IGameState
class IBasicGameState
{
private:
    //IStateManager* getStateManager() const { return static_cast<IStateManager*>(_engine); };
//    IStateManager* getStateManager() const;
    const std::string Name;



public:


    const bool CloneScene;
    const bool Draw2D;
    const bool Draw3D;
    const bool RenderSavedScreenshot;
    const bool ShowCursor;    //!< Show cursor or not



    // { return Scene;};

    virtual ~IBasicGameState();


    const std::string& getName() const
    {
        return Name;
    }






    //inline EffectHandler* effect() { return _engine->_postProcessingEffect; };

//
//    void pushState(IBasicGameState* ) const;
//
//    void replaceThisState(IBasicGameState* state) {
//        popThisState();
//        pushState( state ) ;
//    }
//    void popThisState();

//    virtual void onEscapePress() {};

    virtual void onPush() {};
    virtual void onPop() {};

    virtual void onResume() {};
    virtual void onHold() {};

//    virtual void onPop() {};
//    virtual void onPush() {};

// TODO supprimer a terme

	virtual void Draw() = 0;
	virtual void onUpdate(const TTimeUnit& elapsedTime) = 0;

    virtual void onDebug(const bool& on) {};
//    virtual void onDebugDisactivation() {};

protected:
    /**
     * \brief
     *
     * bool showCursor, cloneOcttree ?
     **/
    IBasicGameState(
            const std::string& name
            , const bool& showCursor
            , const bool& draw2d
            , const bool& draw3d
            , const bool& cloneScene
            , const bool& displayScreenshotAsBackground
            );



private:
    //irr::gui::IGUITab* _rootTab;
    //irr::scene::ISceneManager* _smgr;
    //EffectHandler* _postProcessingHandler;


};





template<class ...VALUES>
class ILoadable
{
public:
    virtual bool load( VALUES... values) = 0;
    virtual void unload() = 0;
};





// TODO renommer en ILoadableGameState
class IGameState :
    public irr::IEventReceiver,
    public IBasicGameState,
    public ILoadable<CGameEngine*,irr::scene::ISceneManager*,irr::video::ITexture*>
{

private:
    CGameEngine* _engine;
    irr::video::ITexture* _backgroundTexture;

protected:
    //! required value: its description
    // std::map < , std::string >
    std::map<
        boost::variant< irr::gui::IGUIElement** ,
                        irr::scene::ISceneNode**,
                        irr::scene::ICameraSceneNode**,
                        irr::scene::ISceneManager**
                        , lua_State**

                         >,
//        std::string
        const char*
        > Required;
//    std::set< boost::variant< irr::gui::IGUIElement*& , irr::scene::ISceneNode*&, irr::scene::ICameraSceneNode*&, irr::scene::ISceneManager*& > > Required;

    irr::gui::IGUIEnvironment* gui() const  ;


    //! Hack should get passed on via load()
    irr::scene::ISceneManager* Scene;
    //! Hack for  State MAnager checks
    irr::gui::IGUIElement* MainTab;

public:



//    EffectHandler* postProcessingFramework();


    IGameState(
            const std::string& name
            , const bool& showCursor
            , const bool& draw2d
            , const bool& draw3d
            , const bool& cloneScene
            , const bool& displayScreenshotAsBackground
            );


    // ou ITexture ?
//    irr::video::IImage* takeScreenshot() const;
//    void takeScreenshot(irr::video::ITexture* );

    inline CGameEngine& engine() const {
//        FUS_ASSERT( _engine != nullptr);
        return *_engine;
    };

    IPostProcessingHandler* pp();

    irr::gui::IGUIElement* getMainTab()
    {
        return MainTab;
    }

    irr::video::ITexture* getBackgroundTexture() const
    {
        return _backgroundTexture;
    }


    bool isLoaded() const;

    irr::scene::ISceneManager* scene() const ;
    irr::video::IVideoDriver* driver() const ;

    virtual bool load(CGameEngine*,irr::scene::ISceneManager*,irr::video::ITexture*) ;
    virtual void unload();


    /** \brief will be displayed when displaying debug data
    **/
    virtual std::wstring generateDebugData() const { return std::move(std::wstring(L"Not implemented") );};

    void showWireframe(const bool&);
};




/*
template<typename STATE>
class IGameStateSingleton
//:
//    public irr::IEventReceiver
//public boost::enable_shared_from_this<IGameState>
{

protected:
    static STATE* Instance;

    IGameStateSingleton() {

    }

public:
    static STATE* instance(){
        if(Instance == nullptr)
        {
            Instance = new STATE();
//            CGameEngine::instance().addToAllocatedStates(Instance);
        }
        return Instance;
    }
// Plus besoin de le passer apres comme ca
// mm pas besoin d'en faire une instance unique ^^

    void destroy()
    {
        if(Instance != nullptr)
        {
            delete Instance;
        }
    }


    virtual ~IGameStateSingleton(){

        //Instance = nullptr;
        destroy();
    };    //!< Compel to inherit fromthis class


};
*/

//typedef IBasicGameState IBasicGameState;


#endif
